"""Pipecat plugin for Faseeh Arabic text-to-speech."""

from pipecat_plugins_faseeh.tts import FaseehTTSService
from pipecat_plugins_faseeh.version import __version__

__all__ = ["FaseehTTSService", "__version__"]
